<?php
namespace Modules\Cadastros\Controllers;

use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;

class Ger11300Periodicidade 
{
    private $environment;
    private $entityManager;
    protected $crudService;
    private $periodicidadeService;

    public function __construct($environment, $entityManager, $crudService, $periodicidadeService)
    {
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->crudService  = $crudService;
        $this->periodicidadeService = $periodicidadeService;
    }
    
    
    /**
     * Função responsável por excluir todos os
     * dias de entrega de uma periodicidade.
     * 
     * @param {DTO\Request} $request Request contendo os dados
     *  da periodicidade.
     * @param {DTO\Response} $response Objeto de resposta
     *  ao usuário.
     */
    public function deleteAllDiaEntrega(DTO\Request $request, DTO\Response $response)
    {
        try {
            $cdPeriEntr = $request->getRow()->get('CDPERIENTR');
            $this->periodicidadeService->deleteAllDiaEntrega($cdPeriEntr);
        } catch(\Exception $e) {
            throw $e;
        }
    }


    /**
     * Função responsável por verificar se existe
     * dias de entrega em uma periodicidade.
     * 
     * @param {DTO\Request} $request Request contendo os dados
     *  da periodicidade.
     * @param {DTO\Response} $response Objeto de resposta
     *  ao usuário.
     */
    public function existsDiaEntrega(DTO\Request $request, DTO\Response $response)
    {
        try{
            $cdPeriEntr = $request->getRow()->get('CDPERIENTR');
            $existDiaEntrega = $this->periodicidadeService->existsDiaEntrega($cdPeriEntr);

            $response->addDataSet(new DataSet('EXIST', $existDiaEntrega));
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    
    /**
     * Função responsável por verificar os paramêtros e retornar
     * os dias de consumo da periodicade informada
     * 
     * @param {DTO\Request} $request Request contendo os dados
     *  da periodicidade.
     * @param {DTO\Response} $response Objeto de resposta
     *  ao usuário.
     */
    public function getDiasConsumo(DTO\Request $request, DTO\Response $response)
    {
        try{
            $row = $request->getRow();
            $diasConsumo = $this->periodicidadeService->getDiasConsumo($row);

            $response->addDataSet(new DataSet('res', $diasConsumo));
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function insertDiasConsumo(DTO\Request $request, DTO\Response $response)
    {
        try{
            
            $row = $request->getRow();
            
            $this->periodicidadeService->deleteDiaConsumo($row);
            
            if ($row['UTPERCONS']) {
                self::trataParametrosInsertConsumo(
                    $row,
                    [ "columnName" => "CDPERIENTR", "value" => $row['CDPERIENTR'], "operator" => "=" ]
                );    
            } else {
                if ($row['__is_new'] && empty($this->periodicidadeService->getDiaEntrega($row)))
                    $this->periodicidadeService->insertDiaEntrega($row);
                else 
                    $this->periodicidadeService->updateDiaEntrega($row);
            }
            
        } catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function trataParametrosInsertConsumo($row, $conditions)
    {
        if (empty($this->periodicidadeService->getDiaEntrega($row))) {
            $this->periodicidadeService->insertDiaEntrega($row);
        }
        
        switch (self::validaTipoConsumo($conditions)) {
            case 'S':
                $row['IDDIACONSINIANT'] = null;
                $row['IDDIACONSFINANT'] = null;
                
                foreach ($row['DIASCONSUMO'] as $dia) {
                    $this->periodicidadeService->insertDiasConsumo(
                        $row,
                        self::setConsumoInicialFinal($dia['DIACONSUMOINI'], $dia['DIACONSUMOFIN'])
                    );
                }
                break;
                
            case 'M':
                $row['IDDIACONSINIANT'] = self::setConsumoMesAnteriorAtual($row['IDDIACONSINIANT']);
                $row['IDDIACONSFINANT'] = self::setConsumoMesAnteriorAtual($row['IDDIACONSFINANT']);
            
                self::validaConsumoInicialFinal(
                    $row['DIACONSUMOINI'], 
                    $row['DIACONSUMOFIN'], 
                    $row['IDDIACONSINIANT'],
                    $row['IDDIACONSFINANT']
                );
                
                $this->periodicidadeService->insertDiasConsumo(
                    $row,
                    self::setConsumoInicialFinal($row['DIACONSUMOINI'], $row['DIACONSUMOFIN'])
                );
                break;
            
            case 'E':
                throw new \Exception("NO_PERMISSION_INSERT_CONSUM_DAYS");
                break;
        }
    }
    
    public function setConsumoInicialFinal($consumoInicial, $consumoFinal)
    {
        return array(
            "DIACONSUMOINI" => $consumoInicial,
            "DIACONSUMOFIN" => $consumoFinal
        );
    }
    
    public function validaTipoConsumo($conditions, $gravaDataConsumo = 'S')
    {
        $diasConsumo = $this->crudService->getFilteredDataSet('perientr', [$conditions])->getRows();

        if (empty($diasConsumo)) {
            return $diasConsumo;
        }
        
        $diasConsumo = $diasConsumo[0];
        
        return $diasConsumo['IDGRAVADTCONSUM'] == $gravaDataConsumo ? 
            $diasConsumo['IDTIPODIA'] : 'E';
    }
    
    public function validaConsumoInicialFinal(
        $consumoInicial, 
        $consumoFinal, 
        $isMesAnteriorInicial = null, 
        $isMesAnteriorFinal = null
    ) {
        $message = "INITIAL_CONSUM_MUST_BE_LESS";
        
        if ($isMesAnteriorInicial && $isMesAnteriorFinal) {
            
            if ($isMesAnteriorInicial == 'N' && $isMesAnteriorFinal == 'S')
                throw new \Exception($message);
            
            if ($isMesAnteriorInicial == $isMesAnteriorFinal)
                if ($consumoInicial > $consumoFinal)
                    throw new \Exception($message);
        } else {
            if ($consumoInicial > $consumoFinal)
                throw new \Exception($message);
        }
    }
    
    public function setConsumoMesAnteriorAtual($isMesAnterior)
    {
        return $isMesAnterior == 1 || $isMesAnterior == 'N' ? 'N' : 'S';
    }
    
    public function savePeriodicidade(DTO\Request $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            
            if ($row['__is_new']) {
                self::verificaExistePeriodicidade($row);
                
                $this->periodicidadeService->insertPeriodicidade($row);
            } else {
                $periodicidade = $this->periodicidadeService->getPeriodicidade($row);
                
                if (self::validaGravaConsumo($row)) {
                    $this->periodicidadeService->deleteAllDiasConsumo($row);
                    $this->periodicidadeService->deleteAllDiaEntrega($row['CDPERIENTR']);
                }
                
                $this->periodicidadeService->updatePeriodicidade($row);
            }
            
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function verificaExistePeriodicidade($row)
    {
        if (!empty($this->periodicidadeService->getPeriodicidade($row))) {
            throw new \Exception("PERIODICIDADE_ALREADY_REGISTERED");
        }
    }
    
    public function validaGravaConsumo($row)
    {
        $periodicidade = $this->periodicidadeService->getPeriodicidade($row)[0];
        
        return $periodicidade['CDPERIENTR'] == $row['CDPERIENTR'] 
            && $periodicidade['IDGRAVADTCONSUM'] != $row['IDGRAVADTCONSUM'];
    }
    
    public function deleteDiaEntrega(DTO\Request $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            
            $this->periodicidadeService->deleteDiaConsumo($row);
            
            $this->periodicidadeService->deleteDiaEntrega($row);
            
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
}
